/*

*/

import java.awt.event.*;

/**
   This class has the responsibility of responding to
   key events by updating the state (data) of our
   graphics program and then notifying the graphics
   part of program that it needs to redraw itself (using
   the new state information).

   The methods from the KeyListener interface will be
   called by the Event Dispatch Thread (the "GUI thread")
   whenever the program's user causes the appropriate event.
*/
public class CircleKeyListener implements KeyListener
{
   private final CircleState cs;
   private final CirclePanel cp;

   public CircleKeyListener(CircleState cs, CirclePanel cp)
   {
      this.cs = cs;
      this.cp = cp;
   }


   // implement the KeyListener interface (three methods)
   @Override public void keyPressed(KeyEvent e)
   {
      final int keyCode = e.getKeyCode();
      if (keyCode == KeyEvent.VK_UP)
      {
         cs.yNew -= 5;   // update the state
         cp.repaint();   // notify the view
      }
      else if (keyCode == KeyEvent.VK_DOWN)
      {
         cs.yNew += 5;
         cp.repaint();
      }
      else if (keyCode == KeyEvent.VK_LEFT)
      {
         cs.xNew -= 5;
         cp.repaint();
      }
      else if (keyCode == KeyEvent.VK_RIGHT)
      {
         cs.xNew += 5;
         cp.repaint();
      }
   }
   @Override public void keyReleased(KeyEvent e){ }
   @Override public void keyTyped(KeyEvent e){ }
}
