/*

*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
   This class builds the GUI, instantiates
   all the program objects, and makes sure
   that every object has a reference to any
   other object that it must communicate with.

   This class instantiates the JFrame that
   is the source of the key events.
*/
public class CircleProgramWithMouse
{
   /**
      This constructor instantiates the GUI.
   */
   public CircleProgramWithMouse()
   {
      final JFrame jf = new JFrame("Circle Program With Mouse");
      jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      final CircleState cs = new CircleState();
      final CirclePanel cp = new CirclePanel(cs);
      jf.getContentPane().setLayout(new FlowLayout());
      //final JButton jb1 = new JButton("Reset1");
      //final JButton jb2 = new JButton("Reset2");
      jf.getContentPane().add(cp);
      //jf.getContentPane().add(jb1);
      //jf.getContentPane().add(jb2);
      jf.pack();
      jf.setLocationRelativeTo(null);
      jf.requestFocusInWindow();
      jf.setVisible(true);

      jf.addKeyListener(new CircleKeyListener(cs, cp));
    //jf.addMouseListener(new CircleMouseListener(cs, cp));
    //jf.addMouseMotionListener(new CircleMouseMotionListener(cs));
      cp.addMouseListener(new CircleMouseListener(cs, cp));
      cp.addMouseMotionListener(new CircleMouseMotionListener(cs));
   }//constructor

   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> new CircleProgramWithMouse()
      );
   }//main()
}

/*
Here is a picture showing some relevant objects in the JVM's heap.

    JFrame object
+----------------------+
|                      |    List<KeyListener>     CircleKeyListener object
|  List<KeyListener>---|------->+------+        +-------------------------+
|  List<MouseListener> |        |      |        | CircleState cs          |
|                      |        +------+        | CirclePanel cp          |
|  addKeyListener()    |        |  ----|------->|                         |
+----------------------+        +------+        | keyPressed(KeyEvent e)  |
GUI object that receives        |      |        | keyReleased(KeyEvent e) |
KeyEvent objects from           +------+        | keyTyped(KeyEvent e)    |
the JVM.                                        |                         |
                                                +-------------------------+
                                                 An object that implements
                                                 the KeyListener interface.
    KeyEvent object
  +-----------------+
  |  information    |
  |  from the OS    |      CircleState object         CirclePanel object
  |  about the      |      +-----------------+       +---------------------------+
  |  keyboard event |      | int xNew        |       | CircleState cs            |
  +-----------------+      | int yNew        |       |                           |    List<MouseListener>
                           | int xOld        |       | List<MouseListener>-------|------->+------+
                           | int yOld        |       | List<MouseMotionListener>-|--+     |  ----|---------------+
   MouseEvent object       | boolean inside  |       |                           |  |     +------+               |
  +-----------------+      +-----------------+       | paintComponent()          |  |                            |
  |  information    |                                +---------------------------+  | List<MouseMotionListener>  |
  |  from the OS    |                                GUI object that needs          +---->+------+               |
  |  about the      |                                to be repainted after                |  ----|---+           |
  |  mouse event    |                                an event occurs.                     +------+   |           |
  +-----------------+                                                                                |           |
                                                                                                     |           |
                           CircleMouseListener object           CircleMouseMotionListener object     |           |
                           +---------------------------+        +---------------------------+        |           |
                           | CircleState cs            |        | CircleState cs            |        |
                           |                           |        |                           |<-------+
                           |                           |        |                           |
                           | mousePressed(MouseEvent e)|        | mouseDragged(MouseEvent e)|
                           | mouseRelease(MouseEvent e)|        +---------------------------+
                           +---------------------------+        An object that implements the
                            An object that implements           MouseMotionListener interface.
                            the MouseListener interface.

*/